void setup(){
  size(pdfLength, pdfHeight, P2D);
  fill(0);
  hint(ENABLE_NATIVE_FONTS);
  fontTitle = createFont("Botanika-Bold", 26);
  font = createFont("Botanika-Bold", 12);
 
  PGraphicsPDF pdf=(PGraphicsPDF)beginRecord(PDF,"TIMELINE.pdf");

  loadDates();
  orderComposers();
  loadPTDates();
  orderPTComposers();
  loadTechDates();
  orderTechnologies();
  loadLabelDates();
  orderLabels();
  loadProjectsDates();
  orderProjects();

  drawGrid();

  stroke(0);
  textFont(fontTitle, 26);
  textAlign(LEFT);
  text("A SKETCH FOR A CONTEMPORARY / EXPERIMENTAL MUSIC TIMELINE", 10, 45);
  textFont(font, 12);
  text("by André Gonçalves . 2010", 10, 55);
  
  // COMPOSERS
  
  text("COMPOSERS / PERFORMERS", 10, topHeight+15);
  try {
    line(0,topHeight,pdfLength,topHeight);
    for(int i=0;i<composerCount;i++){
      int X=int((BirthYear[order[i]]-startYear)*yearPx);
      int Y=int(i*lineInterval+topHeight+lineInterval);
      pushMatrix(); 
      translate(X, Y);
      fill(100);
      stroke(100);
      rect(0,lineHeight/2*-1, ((ActiveYear[order[i]]-startYear)-(BirthYear[order[i]]-startYear))*yearPx, lineHeight);
      fill(0);
      stroke(0);
      rect(((ActiveYear[order[i]])-(BirthYear[order[i]]))*yearPx,lineHeight/2*-1, ((DeathYear[order[i]]-startYear)-(ActiveYear[order[i]]-startYear))*yearPx, lineHeight);
      popMatrix();
      String composerInfo;
      if(DeathYear[order[i]]==ActualYear) composerInfo=ComposerNames[order[i]]+" ("+int(BirthYear[order[i]])+")";
      else composerInfo=ComposerNames[order[i]]+" ("+int(BirthYear[order[i]])+"-"+DeathYear[order[i]]+")";
      Color linkCol = new Color(0,0,0); // Between 0.0 and 1.0
      Chunk chk=new Chunk(composerInfo,FontFactory.getFont("data/Akagi-SB.ttf", 14, com.lowagie.text.Font.NORMAL, linkCol));
      //chk.setBackground(new Color(0,0,0),5,5,5,5);
      chk.setAnchor(ComposerLink[order[i]]);
      // this is using a hacked version of PGraphicsPDF to get access to the PDF content
      PdfContentByte cb=pdf.getContent();
      ColumnText ct = new ColumnText(cb);
      int WIDth=int(chk.getWidthPoint())+20;
      // setSimpleColumn( the phrase, leftmargin coordinate, bottommargin coordinate, box width, box height, line height, alignment )
      ct.setSimpleColumn(new Phrase(chk), X-WIDth,pdfHeight-(Y+lineHeight/2-1), X+600, 50, 0, Element.ALIGN_LEFT);
      ct.go(); 
    }
  }
  catch(Exception e) {
    e.printStackTrace();
  }


  // PT COMPOSERS
  try { 
    line(0,ptComposerHeight,pdfLength,ptComposerHeight);
    textAlign(LEFT);
    text("PORTUGUESE COMPOSERS", 10, ptComposerHeight-5);
    for(int i=0;i<composerPTCount;i++){
      pushMatrix(); 
      translate((PTBirthYear[PTorder[i]]-startYear)*yearPx, i*lineInterval+ptComposerHeight+lineInterval);
      rect(0,lineHeight/2*-1, ((PTDeathYear[PTorder[i]]-startYear)-(PTBirthYear[PTorder[i]]-startYear))*yearPx, lineHeight);
      popMatrix();
      String PTcomposerInfo;
      if(PTDeathYear[PTorder[i]]==ActualYear){
        PTcomposerInfo=ComposerPTNames[PTorder[i]]+" ("+int(PTBirthYear[PTorder[i]])+")";
      }
      else{
        PTcomposerInfo=ComposerPTNames[PTorder[i]]+" ("+int(PTBirthYear[PTorder[i]])+"-"+PTDeathYear[PTorder[i]]+")";
      }
      Color linkCol = new Color(0,0,0); // Between 0.0 and 1.0
      Chunk chk=new Chunk(PTcomposerInfo,FontFactory.getFont("data/Akagi-SB.ttf", 14, com.lowagie.text.Font.NORMAL, linkCol));
      chk.setAnchor(ComposerPTLink[PTorder[i]]);
      // this is using a hacked version of PGraphicsPDF to get access to the PDF content
      PdfContentByte cb=pdf.getContent();
      ColumnText ct = new ColumnText(cb);
      int WIDth=int(chk.getWidthPoint());
      int X=int((PTBirthYear[PTorder[i]]-startYear)*yearPx);
      // setSimpleColumn( the phrase, leftmargin coordinate, bottommargin coordinate, box width, box height, line height, alignment )
      ct.setSimpleColumn(new Phrase(chk), X-WIDth-20,pdfHeight-(i*lineInterval+ptComposerHeight+lineInterval+lineHeight/2-1), X+600, 150, 0, Element.ALIGN_LEFT);
      ct.go(); 


      //generate table body TO ROTATE TEXT
      /*
PdfPTable tableMainHeader = new PdfPTable(1);
       tableMainHeader.setWidthPercentage(100f);
       tableMainHeader.setWidths(new int[] { 15 });
       PdfPCell cell = new PdfPCell();
       
       cell = new PdfPCell();
       //cell.setBackgroundColor(textColor);
       cell.setRotation(90);
       //cell.setBorderColor(0.5);
       cell.setMinimumHeight(30f);
       cell.addElement(chk);
       tableMainHeader.addCell(cell);
       
       ct.addElement(tableMainHeader); 
       ct.go(); 
       
       */
    }
  }
  catch(Exception e) {
    e.printStackTrace();
  }


  // PROJECTS
  try { 
    line(0,projectsHeight,pdfLength,projectsHeight);
    textAlign(LEFT);
    text("MUSICAL PROJECTS", 10, projectsHeight-5);
    for(int i=0;i<projectsCount;i++){
      pushMatrix(); 
      translate((projectBirthYear[projectsOrder[i]]-startYear)*yearPx, i*lineInterval+projectsHeight+lineInterval);
      rect(0,lineHeight/2*-1, ((projectDeathYear[projectsOrder[i]]-startYear)-(projectBirthYear[projectsOrder[i]]-startYear))*yearPx, lineHeight);
      String c;
      if(projectDeathYear[projectsOrder[i]]==ActualYear){
        c=ProjectsNames[projectsOrder[i]]+" ("+int(projectBirthYear[projectsOrder[i]])+")";
      }else{
        c=ProjectsNames[projectsOrder[i]]+" ("+int(projectBirthYear[projectsOrder[i]])+"-"+projectDeathYear[projectsOrder[i]]+")";
      }
      Color linkCol = new Color(0,0,0); // Between 0.0 and 1.0
      Chunk chk=new Chunk(c,FontFactory.getFont("data/Akagi-SB.ttf", 14, com.lowagie.text.Font.NORMAL, linkCol));
      chk.setAnchor(ProjectsLink[projectsOrder[i]]);
      // this is using a hacked version of PGraphicsPDF to get access to the PDF content
      PdfContentByte cb=pdf.getContent();
      ColumnText ct = new ColumnText(cb);
      int WIDth=int(chk.getWidthPoint());
      int X=int((projectBirthYear[projectsOrder[i]]-startYear)*yearPx);
      // setSimpleColumn( the phrase, leftmargin coordinate, bottommargin coordinate, box width, box height, line height, alignment )
      ct.setSimpleColumn(new Phrase(chk), X-WIDth-20,pdfHeight-(i*lineInterval+projectsHeight+lineInterval+lineHeight/2-1), X+600, 50, 0, Element.ALIGN_LEFT);
      ct.go(); 
      popMatrix();
    }
  }catch(Exception e) {
    e.printStackTrace();
  }

/*
  // TECHNOLOGIES
  try { 
    line(0,techHeight,pdfLength,techHeight);
    textAlign(LEFT);
    text("TECHNOLOGY ADVANCES", 10, techHeight-5);
    for(int i=0;i<techCount;i++){
      pushMatrix(); 
      translate(int(techYear[techOrder[i]]-startYear)*yearPx, techHeight);
      rect(-3,lineHeight/2*-1, 6, lineHeight);
      popMatrix();
      String c=TechNames[techOrder[i]];
      int mod=(int)techYear[techOrder[i]];
      int plus=int(techYear[techOrder[i]] % mod * 1200);
      Color linkCol = new Color(0,0,0); // Between 0.0 and 1.0
      Chunk chk=new Chunk(c,FontFactory.getFont("data/Akagi-SB.ttf", 12, com.lowagie.text.Font.NORMAL, linkCol));
      chk.setAnchor(TechLinks[techOrder[i]]);
      // this is using a hacked version of PGraphicsPDF to get access to the PDF content
      PdfContentByte cb=pdf.getContent();
      ColumnText ct = new ColumnText(cb);
      int WIDth=int(chk.getWidthPoint());
      int X=int(techYear[techOrder[i]]-startYear)*yearPx+plus;
      // setSimpleColumn( the phrase, leftmargin coordinate, bottommargin coordinate, box width, box height, line height, alignment )
      ct.setSimpleColumn(new Phrase(chk), X-10,pdfHeight-techHeight-6, X+30, pdfHeight-techHeight-600, 0, Element.ALIGN_LEFT);

      //generate table body TO ROTATE TEXT
      PdfPTable tableTechHeader = new PdfPTable(1);
      tableTechHeader.setWidthPercentage(70f);
      tableTechHeader.setTotalWidth(290f);

      PdfPCell techCell = new PdfPCell();
      techCell = new PdfPCell();
      techCell.setPadding(0);
      techCell.setNoWrap(false);
      //techCell.setBorderWidth(0);
      //techCell.setBackgroundColor(new Color(0.7,0.3,0.7));
      techCell.setRotation(270);
      techCell.setBorder(0);
      //techCell.setBorderColor(new Color(1.0,1.0,1.0));
      //techCell.setMinimumHeight(100f);
      techCell.addElement(chk);
      tableTechHeader.addCell(techCell);

      ct.addElement(tableTechHeader); 
      ct.go(); 

    }
  }
  catch(Exception e) {
    e.printStackTrace();
  }
*/

  exportPDF();

  endRecord();
  println("done");
  exit();  // Quit the program
}




